@@ -6,13 +6,14 @@ package main
 
 import (
 	"bufio"
+	"context"
 	"crypto/tls"
-	"errors"
 	"fmt"
 	"io"
 	"log"
 	"net"
 	"os"
+	"os/signal"
 	"strings"
 	"time"
 
@@ -26,7 +27,6 @@ type configuration struct {
 	jserver  string
 	port     string
 	password string
-	resource string
 	alias    string
 }
 
@@ -44,7 +44,7 @@ func readMessage(messageFilePath string) (string, error) {
 
 	// Open message file.
 	file, err := os.Open(messageFilePath)
-	if errors.Unwrap(err) != nil {
+	if err != nil {
 		return output, fmt.Errorf("readMessage: %w", err)
 	}
 	scanner := bufio.NewScanner(file)
@@ -57,14 +57,14 @@ func readMessage(messageFilePath string) (string, error) {
 		}
 	}
 
-	if err := scanner.Err(); errors.Unwrap(err) != nil {
-		if errors.Unwrap(err) != io.EOF {
+	if err := scanner.Err(); err != nil {
+		if err != io.EOF {
 			return "", fmt.Errorf("readMessage: %w", err)
 		}
 	}
 
 	err = file.Close()
-	if errors.Unwrap(err) != nil {
+	if err != nil {
 		fmt.Println("error while closing file:", err)
 	}
 
@@ -78,10 +78,10 @@ func main() {
 	}
 
 	var (
-		err                                              error
-		message, user, server, password, resource, alias string
-		oxPrivKey                                        *crypto.Key
-		recipients                                       []recipientsType
+		err                                    error
+		message, user, server, password, alias string
+		oxPrivKey                              *crypto.Key
+		recipients                             []recipientsType
 	)
 
 	// Define command line flags.
@@ -93,8 +93,6 @@ func main() {
 	flagPassword := getopt.StringLong("password", 'p', "", "Password for XMPP account.")
 	flagChatroom := getopt.BoolLong("chatroom", 'c', "Send message to a chatroom.")
 	flagDirectTLS := getopt.BoolLong("tls", 't', "Use direct TLS.")
-	flagResource := getopt.StringLong("resource", 'r', "", "Set resource. "+
-		"When sending to a chatroom this is used as 'alias'. DEPRECATED: Use --alias instead.")
 	flagAlias := getopt.StringLong("alias", 'a', "", "Set alias/nickname"+
 		"for chatrooms.")
 	flagFile := getopt.StringLong("file", 'f', "", "Set configuration file. (Default: "+
@@ -152,6 +150,8 @@ func main() {
 			" http upload.")
 	case *flagOx && *flagOOBFile != "":
 		log.Fatal("No encryption possible for OOB data.")
+	case *flagOx && *flagHeadline:
+		log.Fatal("No Ox support for headline messages.")
 	case *flagHeadline && *flagChatroom:
 		log.Fatal("Can't use message type headline for groupchat messages.")
 	}
@@ -170,14 +170,13 @@ func main() {
 	if *flagUser == "" || *flagPassword == "" {
 		// Read configuration from file.
 		config, err := parseConfig(*flagFile)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal("Error parsing ", *flagFile, ": ", err)
 		}
 		// Set connection options according to config.
 		user = config.username
 		server = config.jserver
 		password = config.password
-		resource = config.resource
 		alias = config.alias
 		if config.port != "" {
 			server = net.JoinHostPort(server, fmt.Sprint(config.port))
@@ -200,14 +199,6 @@ func main() {
 	}
 
 	switch {
-	// Use the resource as alias if no alias is specified otherwise but a resource is specified.
-	// Backward compat, will be removed when resource is removed.
-	case alias == "" && *flagAlias == "" && *flagResource != "":
-		alias = *flagResource
-		// Use the resource as alias if no alias is specified otherwise but a resource is specified.
-		// Backward compat, will be removed when resource is removed.
-	case alias == "" && *flagAlias == "" && resource != "":
-		alias = resource
 	// Use "go-sendxmpp" if no nick is specified via config or command line flag.
 	case alias == "" && *flagAlias == "":
 		alias = "go-sendxmpp"
@@ -216,15 +207,6 @@ func main() {
 		alias = *flagAlias
 	}
 
-	// Overwrite resource if specified via command line flag.
-	if *flagResource != "" {
-		fmt.Println("Deprecated flag: --resource.")
-		resource = *flagResource
-	} else if resource == "" {
-		// Use "go-sendxmpp" plus a random string if no other resource is specified.
-		resource = "go-sendxmpp." + getShortID()
-	}
-
 	// Timeout
 	timeout := time.Duration(*flagTimeout) * time.Second
 
@@ -252,7 +234,7 @@ func main() {
 		Host:        server,
 		User:        user,
 		DialTimeout: timeout,
-		Resource:    resource,
+		Resource:    "go-sendxmpp." + getShortID(),
 		Password:    password,
 		// NoTLS doesn't mean that no TLS is used at all but that instead
 		// of using an encrypted connection to the server (direct TLS)
@@ -269,7 +251,7 @@ func main() {
 	// Read message from file.
 	if *flagMessageFile != "" {
 		message, err = readMessage(*flagMessageFile)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 	}
@@ -290,7 +272,7 @@ func main() {
 		}
 
 		if err := scanner.Err(); err != nil {
-			if errors.Unwrap(err) != io.EOF {
+			if err != io.EOF {
 				log.Fatal(err)
 			}
 		}
@@ -307,20 +289,20 @@ func main() {
 
 	// Connect to server.
 	client, err := connect(options, *flagDirectTLS)
-	if errors.Unwrap(err) != nil {
+	if err != nil {
 		log.Fatal(err)
 	}
 
 	iqc := make(chan xmpp.IQ, defaultBufferSize)
 	msgc := make(chan xmpp.Chat, defaultBufferSize)
-	go rcvStanzas(client, iqc, msgc)
-
+	ctx, cancel := context.WithCancel(context.Background())
+	go rcvStanzas(client, iqc, msgc, ctx)
 	for _, r := range getopt.Args() {
 		var re recipientsType
 		re.Jid = r
 		if *flagOx {
 			re.OxKeyRing, err = oxGetPublicKeyRing(client, iqc, r)
-			if errors.Unwrap(err) != nil {
+			if err != nil {
 				re.OxKeyRing = nil
 				fmt.Println("ox: error fetching key for", r+":", err)
 			}
@@ -331,7 +313,7 @@ func main() {
 	// Check that all recipient JIDs are valid.
 	for i, recipient := range recipients {
 		validatedJid, err := MarshalJID(recipient.Jid)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		recipients[i].Jid = validatedJid
@@ -340,52 +322,52 @@ func main() {
 	switch {
 	case *flagOxGenPrivKeyX25519:
 		validatedOwnJid, err := MarshalJID(user)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		err = oxGenPrivKey(validatedOwnJid, client, iqc, *flagOxPassphrase, "x25519")
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		os.Exit(0)
 	case *flagOxGenPrivKeyRSA:
 		validatedOwnJid, err := MarshalJID(user)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		err = oxGenPrivKey(validatedOwnJid, client, iqc, *flagOxPassphrase, "rsa")
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		os.Exit(0)
 	case *flagOxImportPrivKey != "":
 		validatedOwnJid, err := MarshalJID(user)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		err = oxImportPrivKey(validatedOwnJid, *flagOxImportPrivKey,
 			client, iqc)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		os.Exit(0)
 	case *flagOxDeleteNodes:
 		validatedOwnJid, err := MarshalJID(user)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		err = oxDeleteNodes(validatedOwnJid, client, iqc)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		os.Exit(0)
 	case *flagOx:
 		validatedOwnJid, err := MarshalJID(user)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		oxPrivKey, err = oxGetPrivKey(validatedOwnJid, *flagOxPassphrase)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 	}
@@ -400,7 +382,7 @@ func main() {
 		message = validUTF8(*flagOOBFile)
 		// Check if the URI is valid.
 		uri, err := validURI(message)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		message = uri.String()
@@ -423,7 +405,7 @@ func main() {
 			} else {
 				_, err = client.JoinMUCNoHistory(recipient.Jid, alias)
 			}
-			if errors.Unwrap(err) != nil {
+			if err != nil {
 				log.Fatal(err)
 			}
 		}
@@ -434,17 +416,27 @@ func main() {
 			break
 		}
 		// Send raw XML
-		_, err = client.SendOrg(message + "\n")
-		if errors.Unwrap(err) != nil {
+		_, err = client.SendOrg(message)
+		if err != nil {
 			log.Fatal(err)
 		}
 	case *flagInteractive:
 		// Send in endless loop (for usage with e.g. "tail -f").
 		reader := bufio.NewReader(os.Stdin)
+		// Quit if ^C is pressed.
+		c := make(chan os.Signal, 1)
+		signal.Notify(c, os.Interrupt)
+		go func() {
+			for range c {
+				cancel()
+				client.Close()
+				os.Exit(0)
+			}
+		}()
 		for {
 			message, err = reader.ReadString('\n')
 			message = strings.TrimSuffix(message, "\n")
-			if errors.Unwrap(err) != nil {
+			if err != nil {
 				log.Fatal("failed to read from stdin")
 			}
 
@@ -461,13 +453,13 @@ func main() {
 					}
 					oxMessage, err := oxEncrypt(client, oxPrivKey,
 						recipient.Jid, recipient.OxKeyRing, message)
-					if errors.Unwrap(err) != nil {
+					if err != nil {
 						fmt.Println("Ox: couldn't encrypt to",
 							recipient.Jid)
 						continue
 					}
-					_, err = client.SendOrg(oxMessage + "\n")
-					if errors.Unwrap(err) != nil {
+					_, err = client.SendOrg(oxMessage)
+					if err != nil {
 						log.Fatal(err)
 					}
 				default:
@@ -475,7 +467,7 @@ func main() {
 						Remote: recipient.Jid,
 						Type:   msgType, Text: message,
 					})
-					if errors.Unwrap(err) != nil {
+					if err != nil {
 						log.Fatal(err)
 					}
 				}
@@ -488,7 +480,7 @@ func main() {
 			switch {
 			case isOxMsg(v) && *flagOx:
 				msg, t, err := oxDecrypt(v, client, iqc, user, oxPrivKey)
-				if errors.Unwrap(err) != nil {
+				if err != nil {
 					log.Println(err)
 					continue
 				}
@@ -561,7 +553,7 @@ func main() {
 					Remote: recipient.Jid,
 					Type:   msgType, Ooburl: message, Text: message,
 				})
-				if errors.Unwrap(err) != nil {
+				if err != nil {
 					fmt.Println("Couldn't send message to",
 						recipient.Jid)
 				}
@@ -571,7 +563,7 @@ func main() {
 				_, err = client.SendOrg("<message to='" + recipient.Jid + "' type='" +
 					msgType + "'><body>" + message + "</body><x xmlns='jabber:x:oob'><url>" +
 					message + "</url></x></message>")
-				if errors.Unwrap(err) != nil {
+				if err != nil {
 					fmt.Println("Couldn't send message to",
 						recipient.Jid)
 				}
@@ -581,12 +573,12 @@ func main() {
 				}
 				oxMessage, err := oxEncrypt(client, oxPrivKey,
 					recipient.Jid, recipient.OxKeyRing, message)
-				if errors.Unwrap(err) != nil {
+				if err != nil {
 					fmt.Println("Ox: couldn't encrypt to", recipient.Jid)
 					continue
 				}
-				_, err = client.SendOrg(oxMessage + "\n")
-				if errors.Unwrap(err) != nil {
+				_, err = client.SendOrg(oxMessage)
+				if err != nil {
 					log.Fatal(err)
 				}
 			default:
@@ -594,7 +586,7 @@ func main() {
 					Remote: recipient.Jid,
 					Type:   msgType, Text: message,
 				})
-				if errors.Unwrap(err) != nil {
+				if err != nil {
 					log.Fatal(err)
 				}
 			}
@@ -604,5 +596,6 @@ func main() {
 	// Wait for a short time as some messages are not delivered by the server
 	// if the connection is closed immediately after sending a message.
 	time.Sleep(defaultSleepTime * time.Millisecond)
+	cancel()
 	client.Close()
 }
