@@ -1,4 +1,4 @@
-// Copyright 2022 Martin Dosch.
+// Copyright Martin Dosch.
 // Use of this source code is governed by the BSD-2-clause
 // license that can be found in the LICENSE file.
 
@@ -7,9 +7,9 @@ package main
 import (
 	"encoding/base64"
 	"errors"
+	"fmt"
 	"log"
 	"os"
-	"regexp"
 	"runtime"
 	"strings"
 	"time"
@@ -19,23 +19,23 @@ import (
 	"github.com/mattn/go-xmpp"                 // BSD-3-Clause
 )
 
-func oxDeleteNodes(jid string, client *xmpp.Client,
-	iqc chan xmpp.IQ) error {
+func oxDeleteNodes(jid string, client *xmpp.Client, iqc chan xmpp.IQ) error {
 	nodeListRequest := etree.NewDocument()
+	nodeListRequest.WriteSettings.AttrSingleQuote = true
 	query := nodeListRequest.CreateElement("query")
 	query.CreateAttr("xmlns", nsDiscoItems)
 	nlr, err := nodeListRequest.WriteToString()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxDeleteNodes: failed to create node list request %w", err)
 	}
 	iqReply, err := sendIQ(client, iqc, jid, "get", nlr)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxDeleteNodes: failure with node list request iq: %w", err)
 	}
 	nodeListReply := etree.NewDocument()
 	err = nodeListReply.ReadFromBytes(iqReply.Query)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxDeleteNodes: failed to read node list reply iq: %w", err)
 	}
 	query = nodeListReply.SelectElement("query")
 	if query == nil {
@@ -54,10 +54,11 @@ func oxDeleteNodes(jid string, client *xmpp.Client,
 			continue
 		}
 		deleteNodeRequest := etree.NewDocument()
+		deleteNodeRequest.WriteSettings.AttrSingleQuote = true
 		pubsub := deleteNodeRequest.CreateElement("pubsub")
 		pubsub.CreateAttr("xmlns", nsPubsubOwner)
-		delete := pubsub.CreateElement("delete")
-		delete.CreateAttr("node", node.Value)
+		del := pubsub.CreateElement("delete")
+		del.CreateAttr("node", node.Value)
 		dnr, err := deleteNodeRequest.WriteToString()
 		if err != nil {
 			continue
@@ -70,17 +71,15 @@ func oxDeleteNodes(jid string, client *xmpp.Client,
 	return nil
 }
 
-func oxDecrypt(m xmpp.Chat, client *xmpp.Client, iqc chan xmpp.IQ,
-	user string, oxPrivKey *crypto.Key) (string, time.Time, error) {
+func oxDecrypt(m xmpp.Chat, client *xmpp.Client, iqc chan xmpp.IQ, user string, oxPrivKey *crypto.Key) (string, time.Time, error) {
 	var cryptMsgByte []byte
 	var err error
 	sender := strings.Split(m.Remote, "/")[0]
 	for _, r := range m.OtherElem {
 		if r.XMLName.Space == nsOx {
-			cryptMsgByte, err =
-				base64.StdEncoding.DecodeString(r.InnerXML)
+			cryptMsgByte, err = base64.StdEncoding.DecodeString(r.InnerXML)
 			if err != nil {
-				return "error", time.Now(), err
+				return strError, time.Now(), err
 			}
 			break
 		}
@@ -88,55 +87,53 @@ func oxDecrypt(m xmpp.Chat, client *xmpp.Client, iqc chan xmpp.IQ,
 	oxMsg := crypto.NewPGPMessage(cryptMsgByte)
 	keyRing, err := crypto.NewKeyRing(oxPrivKey)
 	if err != nil {
-		return "error", time.Now(), err
+		return strError, time.Now(), err
 	}
 	senderKeyRing, err := oxGetPublicKeyRing(client, iqc, sender)
 	if err != nil {
-		return "error", time.Now(), err
+		return strError, time.Now(), err
 	}
 	decryptMsg, err := keyRing.Decrypt(oxMsg, senderKeyRing, crypto.GetUnixTime())
 	if err != nil {
-		return "error", time.Now(), err
+		return strError, time.Now(), err
 	}
 	// Remove invalid code points.
-	message := strings.ToValidUTF8(string(decryptMsg.Data), "")
-	reg := regexp.MustCompile(`[\x{0000}-\x{0008}\x{000B}\x{000C}\x{000E}-\x{001F}]`)
-	message = reg.ReplaceAllString(message, "")
+	message := validUTF8(string(decryptMsg.Data))
 	doc := etree.NewDocument()
 	err = doc.ReadFromString(message)
 	if err != nil {
-		return "error", time.Now(), err
+		return strError, time.Now(), err
 	}
 	signcrypt := doc.SelectElement("signcrypt")
 	if signcrypt == nil {
-		return "error", time.Now(), errors.New("ox: no signcrypt element")
+		return strError, time.Now(), errors.New("ox: no signcrypt element")
 	}
 	to := signcrypt.SelectElement("to")
 	if to == nil {
-		return "error", time.Now(), errors.New("ox: no to element")
+		return strError, time.Now(), errors.New("ox: no to element")
 	}
 	jid := to.SelectAttr("jid")
 	if jid == nil {
-		return "error", time.Now(), errors.New("ox: no jid attribute")
+		return strError, time.Now(), errors.New("ox: no jid attribute")
 	}
 	if strings.Split(jid.Value, "/")[0] != user {
-		return "error", time.Now(), errors.New("ox: encrypted for wrong user")
+		return strError, time.Now(), errors.New("ox: encrypted for wrong user")
 	}
 	timestamp := signcrypt.SelectElement("time")
 	if timestamp == nil {
-		return "error", time.Now(), errors.New("ox: no time element")
+		return strError, time.Now(), errors.New("ox: no time element")
 	}
 	stamp := timestamp.SelectAttr("stamp")
 	if stamp == nil {
-		return "error", time.Now(), errors.New("ox: no stamp attribute")
+		return strError, time.Now(), errors.New("ox: no stamp attribute")
 	}
 	msgStamp, err := time.Parse("2006-01-02T15:04:05Z0700", stamp.Value)
 	if err != nil {
-		return "error", time.Now(), err
+		return strError, time.Now(), err
 	}
 	payload := signcrypt.SelectElement("payload")
 	if payload == nil {
-		return "error", time.Now(), errors.New("ox: no payload element")
+		return strError, time.Now(), errors.New("ox: no payload element")
 	}
 	body := payload.SelectElement("body")
 	if body == nil {
@@ -154,8 +151,7 @@ func isOxMsg(m xmpp.Chat) bool {
 	return false
 }
 
-func oxImportPrivKey(jid string, privKeyLocation string, client *xmpp.Client,
-	iqc chan xmpp.IQ) error {
+func oxImportPrivKey(jid string, privKeyLocation string, client *xmpp.Client, iqc chan xmpp.IQ) error {
 	xmppURI := "xmpp:" + jid
 	buffer, err := readFile(privKeyLocation)
 	if err != nil {
@@ -167,11 +163,11 @@ func oxImportPrivKey(jid string, privKeyLocation string, client *xmpp.Client,
 		if err != nil {
 			keyDecoded, err := base64.StdEncoding.DecodeString(buffer.String())
 			if err != nil {
-				return err
+				return fmt.Errorf("oxImportPrivKey: failed to import private key: %w", err)
 			}
 			key, err = crypto.NewKey(keyDecoded)
 			if err != nil {
-				return err
+				return fmt.Errorf("oxImportPrivKey: failed to import private key: %w", err)
 			}
 		}
 	}
@@ -181,27 +177,27 @@ func oxImportPrivKey(jid string, privKeyLocation string, client *xmpp.Client,
 	}
 	pk, err := key.GetPublicKey()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxImportPrivKey: failed to get public key associated to private key: %w", err)
 	}
 	pubKey, err := crypto.NewKey(pk)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxImportPrivKey: failed to get public key associated to private key: %w", err)
 	}
 	fingerprint := strings.ToUpper(pubKey.GetFingerprint())
 	_, err = oxRecvPublicKeys(client, iqc, jid, fingerprint)
 	if err != nil {
 		err = oxPublishPubKey(jid, client, iqc, pubKey)
 		if err != nil {
-			return err
+			return fmt.Errorf("oxImportPrivKey: failed to publish public key: %w", err)
 		}
 	}
 	location, err := oxGetPrivKeyLoc(jid)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxImportPrivKey: failed to determine private key location: %w", err)
 	}
 	keySerialized, err := key.Serialize()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxImportPrivKey: failed to serialize private key: %w", err)
 	}
 	err = oxStoreKey(location,
 		base64.StdEncoding.EncodeToString(keySerialized))
@@ -219,21 +215,21 @@ func oxImportPrivKey(jid string, privKeyLocation string, client *xmpp.Client,
 	}
 	err = oxPublishPubKey(jid, client, iqc, pubKey)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxImportPrivKey: failed to publish public key: %w", err)
 	}
 	return nil
 }
 
-func oxPublishPubKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ,
-	pubKey *crypto.Key) error {
+func oxPublishPubKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ, pubKey *crypto.Key) error {
 	keyCreated := time.Now().UTC().Format("2006-01-02T15:04:05Z")
 	fingerprint := strings.ToUpper(pubKey.GetFingerprint())
 	keySerialized, err := pubKey.Serialize()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxPublishPubKey: failed to serialize pubkey: %w", err)
 	}
 	pubKeyBase64 := base64.StdEncoding.EncodeToString(keySerialized)
 	root := etree.NewDocument()
+	root.WriteSettings.AttrSingleQuote = true
 	pubsub := root.CreateElement("pubsub")
 	pubsub.CreateAttr("xmlns", nsPubsub)
 	publish := pubsub.CreateElement("publish")
@@ -259,13 +255,13 @@ func oxPublishPubKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ,
 	value.CreateText("open")
 	xmlstring, err := root.WriteToString()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxPublishPubKey: failed to create publish public key iq xml: %w", err)
 	}
 	iqReply, err := sendIQ(client, iqc, jid, "set", xmlstring)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxPublishPubKey: iq failure publishing public key: %w", err)
 	}
-	if iqReply.Type != "result" {
+	if iqReply.Type != strResult {
 		return errors.New("error while publishing public key")
 	}
 	ownPubKeyRingFromPubsub, err := oxRecvPublicKeys(client, iqc, jid, fingerprint)
@@ -281,6 +277,7 @@ func oxPublishPubKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ,
 		return errors.New("couldn't successfully verify public key upload")
 	}
 	root = etree.NewDocument()
+	root.WriteSettings.AttrSingleQuote = true
 	pubsub = root.CreateElement("pubsub")
 	pubsub.CreateAttr("xmlns", nsPubsub)
 	publish = pubsub.CreateElement("publish")
@@ -306,13 +303,13 @@ func oxPublishPubKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ,
 	value.CreateText("open")
 	xmlstring, err = root.WriteToString()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxPublishPubKey: failed to create xml for iq to publish public key list: %w", err)
 	}
 	iqReply, err = sendIQ(client, iqc, jid, "set", xmlstring)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxPublishPubKey: iq failure publishing public key list: %w", err)
 	}
-	if iqReply.Type != "result" {
+	if iqReply.Type != strResult {
 		return errors.New("couldn't publish public key list")
 	}
 	return nil
@@ -333,18 +330,18 @@ func oxGetPrivKeyLoc(jid string) (string, error) {
 	default:
 		homeDir, err = os.UserHomeDir()
 		if err != nil {
-			return "error", err
+			return strError, fmt.Errorf("oxGetPrivKeyLoc: failed to determine user dir: %w", err)
 		}
 		if homeDir == "" {
-			return "error", err
+			return strError, errors.New("oxGetPrivKeyLoc: received empty string for home directory")
 		}
 		dataDir = homeDir + "/.local/share"
 	}
-	dataDir = dataDir + "/go-sendxmpp/oxprivkeys/"
+	dataDir += "/go-sendxmpp/oxprivkeys/"
 	if _, err = os.Stat(dataDir); os.IsNotExist(err) {
-		err = os.MkdirAll(dataDir, 0700)
+		err = os.MkdirAll(dataDir, defaultDirRights)
 		if err != nil {
-			return "error", err
+			return strError, fmt.Errorf("oxGetPrivKeyLoc: could not create folder for private keys: %w", err)
 		}
 	}
 	dataFile := dataDir + base64.StdEncoding.EncodeToString([]byte(jid))
@@ -366,18 +363,18 @@ func oxGetPubKeyLoc(fingerprint string) (string, error) {
 	default:
 		homeDir, err = os.UserHomeDir()
 		if err != nil {
-			return "error", err
+			return strError, fmt.Errorf("oxGetPubKeyLoc: failed to determine user dir: %w", err)
 		}
 		if homeDir == "" {
-			return "error", err
+			return strError, errors.New("oxGetPubKeyLoc: received empty string for home directory")
 		}
 		dataDir = homeDir + "/.local/share"
 	}
-	dataDir = dataDir + "/go-sendxmpp/oxpubkeys/"
+	dataDir += "/go-sendxmpp/oxpubkeys/"
 	if _, err = os.Stat(dataDir); os.IsNotExist(err) {
-		err = os.MkdirAll(dataDir, 0700)
+		err = os.MkdirAll(dataDir, defaultDirRights)
 		if err != nil {
-			return "error", err
+			return strError, fmt.Errorf("oxGetPubKeyLoc: could not create folder for public keys: %w", err)
 		}
 	}
 	dataFile := dataDir + fingerprint
@@ -396,11 +393,11 @@ func oxGetPrivKey(jid string, passphrase string) (*crypto.Key, error) {
 	keyString := keyBuffer.String()
 	decodedPrivKey, err := base64.StdEncoding.DecodeString(keyString)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPrivKey: failed to decode private key: %w", err)
 	}
 	key, err := crypto.NewKey(decodedPrivKey)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPrivKey: failed to decode private key: %w", err)
 	}
 	if passphrase != "" {
 		key, err = key.Unlock([]byte(passphrase))
@@ -410,7 +407,7 @@ func oxGetPrivKey(jid string, passphrase string) (*crypto.Key, error) {
 	}
 	isLocked, err := key.IsLocked()
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPrivKey: failed to check whether private key is locked: %w", err)
 	}
 	if isLocked {
 		log.Fatal("Ox: private key is locked.")
@@ -425,41 +422,45 @@ func oxStoreKey(location string, key string) error {
 	var file *os.File
 	file, err := os.Create(location)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxStoreKey: failed to create key location: %w", err)
 	}
-	defer file.Close()
 	if runtime.GOOS != "windows" {
-		_ = file.Chmod(os.FileMode(0600))
+		_ = file.Chmod(os.FileMode(defaultFileRights))
 	} else {
-		_ = file.Chmod(os.FileMode(0200))
+		_ = file.Chmod(os.FileMode(defaultFileRightsWin))
 	}
 	_, err = file.Write([]byte(key))
 	if err != nil {
-		return err
+		return fmt.Errorf("oxStoreKey: failed to write key to file: %w", err)
+	}
+	err = file.Close()
+	if err != nil {
+		fmt.Println("error while closing file:", err)
 	}
 	return nil
 }
 
 func oxGenPrivKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ,
-	passphrase string, keyType string) error {
+	passphrase string, keyType string,
+) error {
 	xmppURI := "xmpp:" + jid
-	key, err := crypto.GenerateKey(xmppURI, "", keyType, 4096)
+	key, err := crypto.GenerateKey(xmppURI, "", keyType, defaultRSABits)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxGenPrivKey: failed to generate private key: %w", err)
 	}
 	if passphrase != "" {
 		key, err = key.Lock([]byte(passphrase))
 		if err != nil {
-			return err
+			return fmt.Errorf("oxGenPrivKey: failed to lock key with passphrase: %w", err)
 		}
 	}
 	keySerialized, err := key.Serialize()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxGenPrivKey: failed to serialize private key: %w", err)
 	}
 	location, err := oxGetPrivKeyLoc(jid)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxGenPrivKey: failed to get private key location: %w", err)
 	}
 	err = oxStoreKey(location,
 		base64.StdEncoding.EncodeToString(keySerialized))
@@ -468,22 +469,22 @@ func oxGenPrivKey(jid string, client *xmpp.Client, iqc chan xmpp.IQ,
 	}
 	decodedPubKey, err := key.GetPublicKey()
 	if err != nil {
-		return err
+		return fmt.Errorf("oxGenPrivKey: failed to decode public key: %w", err)
 	}
 	pubKey, err := crypto.NewKey(decodedPubKey)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxGenPrivKey: failed to decode public key: %w", err)
 	}
 	err = oxPublishPubKey(jid, client, iqc, pubKey)
 	if err != nil {
-		return err
+		return fmt.Errorf("oxGenPrivKey: failed to publish public key: %w", err)
 	}
 	return nil
 }
 
-func oxRecvPublicKeys(client *xmpp.Client, iqc chan xmpp.IQ, recipient string,
-	fingerprint string) (*crypto.KeyRing, error) {
+func oxRecvPublicKeys(client *xmpp.Client, iqc chan xmpp.IQ, recipient string, fingerprint string) (*crypto.KeyRing, error) {
 	opkr := etree.NewDocument()
+	opkr.WriteSettings.AttrSingleQuote = true
 	opkrPs := opkr.CreateElement("pubsub")
 	opkrPs.CreateAttr("xmlns", nsPubsub)
 	opkrPsItems := opkrPs.CreateElement("items")
@@ -491,24 +492,24 @@ func oxRecvPublicKeys(client *xmpp.Client, iqc chan xmpp.IQ, recipient string,
 	opkrPsItems.CreateAttr("max_items", "1")
 	opkrString, err := opkr.WriteToString()
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxRecvPublicKeys: failed to generate xml for public key request: %w", err)
 	}
 	oxPublicKey, err := sendIQ(client, iqc, recipient, "get", opkrString)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxRecvPublicKeys: iq error requesting public keys: %w", err)
 	}
-	if oxPublicKey.Type != "result" {
-		return nil, errors.New("Error while requesting public key for " +
+	if oxPublicKey.Type != strResult {
+		return nil, errors.New("error while requesting public key for " +
 			recipient)
 	}
 	oxPublicKeyXML := etree.NewDocument()
 	err = oxPublicKeyXML.ReadFromBytes(oxPublicKey.Query)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxRecvPublicKeys: failed parsing iq reply to public key request: %w", err)
 	}
 	keyring, err := crypto.NewKeyRing(nil)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxRecvPublicKeys: failed reading public key: %w", err)
 	}
 	oxPublicKeyXMLPubsub := oxPublicKeyXML.SelectElement("pubsub")
 	if oxPublicKeyXMLPubsub == nil {
@@ -533,31 +534,31 @@ func oxRecvPublicKeys(client *xmpp.Client, iqc chan xmpp.IQ, recipient string,
 		}
 		decodedPubKey, err := base64.StdEncoding.DecodeString(data.Text())
 		if err != nil {
-			return nil, err
+			return nil, fmt.Errorf("oxRecvPublicKeys: failed to decode public key: %w", err)
 		}
 		key, err := crypto.NewKey(decodedPubKey)
 		if err != nil {
-			return nil, err
+			return nil, fmt.Errorf("oxRecvPublicKeys: failed to decode public key: %w", err)
 		}
 		if key.IsExpired() {
 			return nil, errors.New("Key is expired: " + fingerprint)
 		}
 		err = keyring.AddKey(key)
 		if err != nil {
-			return nil, err
+			return nil, fmt.Errorf("oxRecvPublicKeys: failed adding public key to keyring: %w", err)
 		}
 	}
 	return keyring, nil
 }
 
-func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
-	recipient string) (*crypto.KeyRing, error) {
+func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ, recipient string) (*crypto.KeyRing, error) {
 	publicKeyRing, err := crypto.NewKeyRing(nil)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPublicKeyRing: failed to create a new keyring: %w", err)
 	}
 
 	oxPubKeyListReq := etree.NewDocument()
+	oxPubKeyListReq.WriteSettings.AttrSingleQuote = true
 	oxPubKeyListReqPs := oxPubKeyListReq.CreateElement("pubsub")
 	oxPubKeyListReqPs.CreateAttr("xmlns", nsPubsub)
 	oxPubKeyListReqPsItems := oxPubKeyListReqPs.CreateElement("items")
@@ -571,20 +572,20 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 	if err != nil {
 		log.Fatal(err)
 	}
-	if oxPublicKeyList.Type != "result" {
-		return nil, errors.New("Error while requesting public openpgp keys for " +
+	if oxPublicKeyList.Type != strResult {
+		return nil, errors.New("error while requesting public openpgp keys for " +
 			recipient)
 	}
 	oxPubKeyListXML := etree.NewDocument()
 	err = oxPubKeyListXML.ReadFromBytes(oxPublicKeyList.Query)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPublicKeyRing: failed to parse answer to public key list request: %w", err)
 	}
 
 	pubKeyRingID := "none"
 	newestKey, err := time.Parse(time.RFC3339, "1900-01-01T00:00:00Z")
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPublicKeyRing: failed to set time for newest key to 1900-01-01: %w", err)
 	}
 
 	oxPubKeyListXMLPubsub := oxPubKeyListXML.SelectElement("pubsub")
@@ -601,10 +602,9 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 	}
 	oxPubKeyListXMLPubsubItemsItemPkl := oxPubKeyListXMLPubsubItemsItem.SelectElement("public-keys-list")
 	if oxPubKeyListXMLPubsubItemsItemPkl == nil {
-		return nil, errors.New("ox: no keblic-keys-list element")
+		return nil, errors.New("ox: no public-keys-list element")
 	}
-	oxPubKeyListXMLPubsubItemsItemPklPm :=
-		oxPubKeyListXMLPubsubItemsItemPkl.SelectElements("pubkey-metadata")
+	oxPubKeyListXMLPubsubItemsItemPklPm := oxPubKeyListXMLPubsubItemsItemPkl.SelectElements("pubkey-metadata")
 	for _, r := range oxPubKeyListXMLPubsubItemsItemPklPm {
 		date := r.SelectAttr("date")
 		if date == nil {
@@ -616,7 +616,7 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 		}
 		keyDate, err := time.Parse(time.RFC3339, date.Value)
 		if err != nil {
-			return nil, err
+			return nil, fmt.Errorf("oxGetPublicKeyRing: failed to parse time stamp for key: %w", err)
 		}
 		if keyDate.After(newestKey) {
 			newestKey = keyDate
@@ -629,7 +629,7 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 
 	pubKeyRingLocation, err := oxGetPubKeyLoc(pubKeyRingID)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPublicKeyRing: failed to get public key ring location: %w", err)
 	}
 
 	pubKeyReadXML := etree.NewDocument()
@@ -639,7 +639,7 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 		if date != nil {
 			savedKeysDate, err := time.Parse(time.RFC3339, date.Text())
 			if err != nil {
-				return nil, err
+				return nil, fmt.Errorf("oxGetPublicKeyRing: failed to parse time for saved key: %w", err)
 			}
 			if !savedKeysDate.Before(newestKey) {
 				pubKeys := pubKeyReadXML.SelectElements("pubkey")
@@ -649,16 +649,16 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 				for _, r := range pubKeys {
 					keyByte, err := base64.StdEncoding.DecodeString(r.Text())
 					if err != nil {
-						return nil, err
+						return nil, fmt.Errorf("oxGetPublicKeyRing: failed to decode saved key: %w", err)
 					}
 					key, err := crypto.NewKey(keyByte)
 					if err != nil {
-						return nil, err
+						return nil, fmt.Errorf("oxGetPublicKeyRing: failed to parse saved key: %w", err)
 					}
 					if !key.IsExpired() {
 						err = publicKeyRing.AddKey(key)
 						if err != nil {
-							return nil, err
+							return nil, fmt.Errorf("oxGetPublicKeyRing: failed to add key to public keyring: %w", err)
 						}
 					}
 				}
@@ -670,7 +670,7 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 	}
 	pubKeyRing, err := oxRecvPublicKeys(client, iqc, recipient, pubKeyRingID)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPublicKeyRing: failed to get public keyring: %w", err)
 	}
 	pubKeySaveXML := etree.NewDocument()
 	date := pubKeySaveXML.CreateElement("date")
@@ -678,29 +678,25 @@ func oxGetPublicKeyRing(client *xmpp.Client, iqc chan xmpp.IQ,
 	for _, key := range pubKeyRing.GetKeys() {
 		keySerialized, err := key.Serialize()
 		if err != nil {
-			return nil, err
-		}
-		if err != nil {
-			return nil, err
+			return nil, fmt.Errorf("oxGetPublicKeyRing: failed to serialize key: %w", err)
 		}
 		saveKey := pubKeySaveXML.CreateElement("pubkey")
 		saveKey.SetText(base64.StdEncoding.EncodeToString(keySerialized))
 	}
 	err = pubKeySaveXML.WriteToFile(pubKeyRingLocation)
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("oxGetPublicKeyRing: failed to create xml for saving public key: %w", err)
 	}
 	return pubKeyRing, nil
 }
 
-func oxEncrypt(client *xmpp.Client, iqc chan xmpp.IQ, oxPrivKey *crypto.Key,
-	recipient string, keyRing *crypto.KeyRing, message string) (string, error) {
+func oxEncrypt(client *xmpp.Client, oxPrivKey *crypto.Key, recipient string, keyRing *crypto.KeyRing, message string) (string, error) {
 	if message == "" {
 		return "", nil
 	}
 	privKeyRing, err := crypto.NewKeyRing(oxPrivKey)
 	if err != nil {
-		return "error", err
+		return strError, fmt.Errorf("oxEncrypt: failed to create private keyring: %w", err)
 	}
 	ownJid := strings.Split(client.JID(), "/")[0]
 	if recipient != ownJid {
@@ -711,6 +707,7 @@ func oxEncrypt(client *xmpp.Client, iqc chan xmpp.IQ, oxPrivKey *crypto.Key,
 		}
 	}
 	oxCryptMessage := etree.NewDocument()
+	oxCryptMessage.WriteSettings.AttrSingleQuote = true
 	oxCryptMessageSc := oxCryptMessage.CreateElement("signcrypt")
 	oxCryptMessageSc.CreateAttr("xmlns", nsOx)
 	oxCryptMessageScTo := oxCryptMessageSc.CreateElement("to")
@@ -725,14 +722,15 @@ func oxEncrypt(client *xmpp.Client, iqc chan xmpp.IQ, oxPrivKey *crypto.Key,
 	oxCryptMessageScPayloadBody.CreateText(message)
 	ocm, err := oxCryptMessage.WriteToString()
 	if err != nil {
-		return "error", err
+		return strError, fmt.Errorf("oxEncrypt: failed to create xml for ox crypt message: %w", err)
 	}
 	plainMessage := crypto.NewPlainMessage([]byte(ocm))
 	pgpMessage, err := keyRing.Encrypt(plainMessage, privKeyRing)
 	if err != nil {
-		return "error", err
+		return strError, fmt.Errorf("oxEncrypt: failed to create pgp message: %w", err)
 	}
 	om := etree.NewDocument()
+	om.WriteSettings.AttrSingleQuote = true
 	omMessage := om.CreateElement("message")
 	omMessage.CreateAttr("to", recipient)
 	omMessage.CreateAttr("id", getID())
@@ -748,7 +746,7 @@ func oxEncrypt(client *xmpp.Client, iqc chan xmpp.IQ, oxPrivKey *crypto.Key,
 	omMessageBody.CreateText(oxAltBody)
 	oms, err := om.WriteToString()
 	if err != nil {
-		return "error", err
+		return strError, fmt.Errorf("oxEncrypt: failed to create xml for ox message: %w", err)
 	}
 
 	return oms, nil
